# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncOffsetPage, AsyncOffsetPage
from ...types.admin import user_connection_list_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.admin.user_connection_response import UserConnectionResponse

__all__ = ["UserConnectionsResource", "AsyncUserConnectionsResource"]


class UserConnectionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UserConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return UserConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return UserConnectionsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        provider: user_connection_list_params.Provider | NotGiven = NOT_GIVEN,
        user: user_connection_list_params.User | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncOffsetPage[UserConnectionResponse]:
        """
        List all auth connections

        Args:
          limit: Page size

          offset: Page offset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/admin/user_connections",
            page=SyncOffsetPage[UserConnectionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "provider": provider,
                        "user": user,
                    },
                    user_connection_list_params.UserConnectionListParams,
                ),
            ),
            model=UserConnectionResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete a user/auth provider connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/admin/user_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncUserConnectionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUserConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#accessing-raw-response-data-eg-headers
        """
        return AsyncUserConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArcadeAI/arcade-py#with_streaming_response
        """
        return AsyncUserConnectionsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        offset: int | NotGiven = NOT_GIVEN,
        provider: user_connection_list_params.Provider | NotGiven = NOT_GIVEN,
        user: user_connection_list_params.User | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[UserConnectionResponse, AsyncOffsetPage[UserConnectionResponse]]:
        """
        List all auth connections

        Args:
          limit: Page size

          offset: Page offset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/admin/user_connections",
            page=AsyncOffsetPage[UserConnectionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "provider": provider,
                        "user": user,
                    },
                    user_connection_list_params.UserConnectionListParams,
                ),
            ),
            model=UserConnectionResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete a user/auth provider connection

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/admin/user_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class UserConnectionsResourceWithRawResponse:
    def __init__(self, user_connections: UserConnectionsResource) -> None:
        self._user_connections = user_connections

        self.list = to_raw_response_wrapper(
            user_connections.list,
        )
        self.delete = to_raw_response_wrapper(
            user_connections.delete,
        )


class AsyncUserConnectionsResourceWithRawResponse:
    def __init__(self, user_connections: AsyncUserConnectionsResource) -> None:
        self._user_connections = user_connections

        self.list = async_to_raw_response_wrapper(
            user_connections.list,
        )
        self.delete = async_to_raw_response_wrapper(
            user_connections.delete,
        )


class UserConnectionsResourceWithStreamingResponse:
    def __init__(self, user_connections: UserConnectionsResource) -> None:
        self._user_connections = user_connections

        self.list = to_streamed_response_wrapper(
            user_connections.list,
        )
        self.delete = to_streamed_response_wrapper(
            user_connections.delete,
        )


class AsyncUserConnectionsResourceWithStreamingResponse:
    def __init__(self, user_connections: AsyncUserConnectionsResource) -> None:
        self._user_connections = user_connections

        self.list = async_to_streamed_response_wrapper(
            user_connections.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            user_connections.delete,
        )
